#include "guipanel.h"
#include "qframe.h"
#include "kiconloader.h"

static long instances = 0;

Gui_PANEL::Gui_PANEL()
{
	Widget = 0;
	ParentWidget = 0;
	instances++;
	printf("NOW %ld instances of the panel active\n",instances);
}

Gui_PANEL::~Gui_PANEL()
{
	instances--;
	printf("NOW %ld instances of the panel active\n",instances);
}

void Gui_PANEL::publish(Arts::ArtsServer *Server, Arts::ModuleBroker *Broker)
{
	Arts::ModuleInfo info;

	info.name = CORBA::string_dup("Gui_PANEL");
	info.isInterface = false;
	info.isStructure = false;

	addPort(info,"width",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"height",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"pixmap",Arts::input,Arts::string_data,Arts::property);

	Broker->publishModule(Server, info);
}

void *Gui_PANEL::Creator()
{
	return new Gui_PANEL;
}

ModuleClient MC_Gui_PANEL(GuiModule::get_MS,"Gui_PANEL",Gui_PANEL::Creator);

void Gui_PANEL::applyParameters(Arts::ModuleDesc *desc)
{
	printf("crw\n");
	if(!Widget)
	{
		QFrame *Frame = new QFrame();
		Frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
		Frame->setLineWidth( 2 );
		Widget = Frame;
	}

	printf("sfs\n");
	Widget->setFixedSize(getFloatParam(desc,"width")*20,
                         getFloatParam(desc,"height")*20);

	string bg = getStringParam(desc,"pixmap");
	if(strlen(bg.c_str()))
	{
		KIconLoader iconloader;
		QString iconname = QString(bg.c_str())+".xpm";
		QPixmap *pixmap = new QPixmap(iconloader.loadIcon(iconname));

		if(!pixmap->height())
		{
			delete pixmap;
			pixmap = 0;
		}
		else
		{
			Widget->setBackgroundPixmap(*pixmap);
		}
	}
}

QWidget *Gui_PANEL::isParent()
{
	return(Widget);
}

void Gui_PANEL::setParent(QWidget *parent, QBoxLayout *layout)
{
	if(Widget)
	{
		printf("recreating widget...\n");
		Widget->recreate(parent,0,QPoint(0,0));
		if(layout)
		{
			printf("adding widget to layout\n");
			layout->addWidget(Widget);
		}
		printf("ok.\n");
	}
	ParentWidget = parent;
}

void Gui_PANEL::start()
{
	Widget->show();
}
