	/*

	Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#include "../synthesizer/debug.cc"
#include "../synthesizer/artsobject.cc"
#include "artsbuilder.h"
#include "guiserver_impl.h"
#include "guimodule.h"

GUIServer_impl::GUIServer_impl(Arts::ModuleBroker *Broker)
{
	describe("GUIServer");

	ModuleBroker = Broker;
	ModuleBroker->incRef();

	_ID = ModuleBroker->mkID();
	nextMID = 1;

	ModuleServer<GuiModule> *MS_GuiModule =
		(ModuleServer<GuiModule> *)GuiModule::get_MS();

	long nr;
	for(nr=0;nr < MS_GuiModule->getModuleCount(); nr++)
	{
		char *name=MS_GuiModule->getModuleName(nr);
		printf("publishing module %s:\n",name);
		if(name)
		{
			GuiModule *m = (GuiModule *)MS_GuiModule->getModule(name);

			printf(" - module created\n");
			if(m)
			{
			    printf(" - not null\n");
				m->publish(this,Broker);
				delete m;
			}
		}
	}	
}

void GUIServer_impl::cleanUp()
{
	ModuleBroker->serverShutdown(this);
	ModuleBroker->decRef();
}

void GUIServer_impl::setGlobalParent(QWidget *globalParent)
{
	this->globalParent = globalParent;
}

void GUIServer_impl::setGlobalLayout(QBoxLayout *globalLayout)
{
	this->globalLayout = globalLayout;
}

/************ GUI Server CORBA functions *************/

CORBA::Long GUIServer_impl::ID()
{
	return _ID;
}

CORBA::Long GUIServer_impl::createModules()
{
	return nextMID++;
}

CORBA::Boolean GUIServer_impl::createModule(CORBA::Long mid,
												Arts::ModuleDesc *desc)
{
	printf("create called\n");
	CORBA::String_var name = desc->Name();
	printf("md->Name() = %s\n",(const char *)name);

	ModuleServer<GuiModule *> *MS_GuiModule = GuiModule::get_MS();

	GuiModule *m = (GuiModule *)MS_GuiModule->getModule(name);
	assert(m);
	m->setMID(mid);
	m->applyParameters(desc);
	modules.push_back(m);
}

CORBA::Boolean GUIServer_impl::localConnectModules(CORBA::Long mid)
{
}

CORBA::Boolean GUIServer_impl::finalizeModules(CORBA::Long mid)
{
	QWidget *localParent = globalParent;
	list<GuiModule *>::iterator i;

	// search the module which will be parent for the others (usually a panel)
	for(i=modules.begin();i != modules.end(); i++)
	{
		GuiModule *m = *i;

		if(m->MID() == mid && m->isParent())
			localParent = m->isParent();
	}

	// reparent the modules to their parents
	for(i=modules.begin();i != modules.end(); i++)
	{
		GuiModule *m = *i;

		if(m->MID() == mid)
		{
			if(m->isParent())
				m->setParent(globalParent,globalLayout);
			else				// localLayout?
				m->setParent(localParent,0);
			m->finalize();
		}
	}
}

void GUIServer_impl::startModules(CORBA::Long mid)
{
	list<GuiModule *>::iterator i;
	printf("start\n");
	for(i=modules.begin();i != modules.end(); i++)
	{
		GuiModule *m = *i;

		if(m->MID() == mid)
			m->start();
	}
}

void GUIServer_impl::deleteModules(CORBA::Long mid)
{
	list<GuiModule *>::iterator i,next;
	printf("deleteModules(%ld)\n",mid);
	for(i=modules.begin();i != modules.end(); i = next)
	{
		GuiModule *m = *i;
		next = i;
		++next;

		if(m->MID() == mid)
		{
			delete m;
			modules.erase(i);
		}
	}
}

CORBA::Boolean GUIServer_impl::remoteConnectModules(CORBA::Long mid,
	CORBA::Long remotemid, const Arts::ModuleDescSeq& modules,
	Arts::ArtsServer *server)
{
}

void GUIServer_impl::requestSignal(CORBA::Long mid, CORBA::Long portid,
	Arts::Receiver *receiver)
{
	list<GuiModule *>::iterator i;
	printf("requestSignal\n");
	for(i=modules.begin();i != modules.end(); i++)
	{
		GuiModule *m = *i;

		if(m->MID() == mid)
			m->assignReceiver(portid,receiver);
	}
}
