#include "guislider.h"
#include "qframe.h"
#include "qbitmap.h"
#include "kiconloader.h"

MovingLabel::MovingLabel(int minx, int miny, int maxx, int maxy,
							Gui_SLIDER *slider) :QLabel()
{
	isMoving = false;
	this->minx = minx;
	this->miny = miny;
	this->maxx = maxx;
	this->maxy = maxy;
	this->slider = slider;
}

void MovingLabel::mousePressEvent(QMouseEvent *event)
{
	if ( event->button() == LeftButton )
	{
		isMoving = true;
		movestart = event->pos();
		sx = x();
		sy = y();
	}
}

void MovingLabel::mouseMoveEvent(QMouseEvent *event)
{
	if(isMoving)
	{
		int dx = event->x() - movestart.x();
		int dy = event->y() - movestart.y();

		int newx = x()+dx;
		int newy = y()+dy;

		if(newx < minx) newx = minx;
		if(newy < miny) newy = miny;
		if(newx > maxx) newx = maxx;
		if(newy > maxy) newy = maxy;

		move(newx,newy);
		
		float mdx = maxx-minx;
		float mdy = maxy-miny;

		if(mdx < 1) mdx = 1;
		if(mdy < 1) mdy = 1;
		slider->sendPosChange((float)(newx-minx)/mdx,(float)(newy-miny)/mdy);
	}
}

void MovingLabel::mouseReleaseEvent(QMouseEvent *event)
{
	if ( event->button() == LeftButton ) isMoving = false;
}

Gui_SLIDER::Gui_SLIDER()
{
	printf("constructor\n");
	Widget = 0;
	SLWidget = 0;
	ParentWidget = 0;
}

void Gui_SLIDER::publish(Arts::ArtsServer *Server, Arts::ModuleBroker *Broker)
{
	Arts::ModuleInfo info;

	info.name = CORBA::string_dup("Gui_SLIDER");
	info.isInterface = false;
	info.isStructure = false;

	addPort(info,"x",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"y",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"bgpixmap",Arts::input,Arts::string_data,Arts::property);
	addPort(info,"slpixmap",Arts::input,Arts::string_data,Arts::property);
	addPort(info,"min",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"max",Arts::input,Arts::audio_data,Arts::property);
	addPort(info,"value",Arts::output,Arts::audio_data,Arts::stream);

	Broker->publishModule(Server, info);
}

void *Gui_SLIDER::Creator()
{
	Gui_SLIDER *result = new Gui_SLIDER;
	return (GuiModule *)result;
}

ModuleClient MC_Gui_SLIDER(GuiModule::get_MS,"Gui_SLIDER",Gui_SLIDER::Creator);

void Gui_SLIDER::applyParameters(Arts::ModuleDesc *desc)
{
	KIconLoader iconloader;

	string bg = getStringParam(desc,"bgpixmap");
	if(strlen(bg.c_str()))
	{
		QString iconname = QString(bg.c_str())+".xpm";
		bgpixmap = iconloader.loadIcon(iconname);
	}

	string sl = getStringParam(desc,"slpixmap");
	if(strlen(sl.c_str()))
	{
		QString iconname = QString(sl.c_str())+".xpm";
		slpixmap = iconloader.loadIcon(iconname);
	}

	x = (int)(getFloatParam(desc,"x")*20);
	y = (int)(getFloatParam(desc,"y")*20);

	minvalue = getFloatParam(desc,"min");
	printf("got parameter minvalue %f\n",minvalue);
	maxvalue = getFloatParam(desc,"max");
	printf("got parameter minvalue %f\n",maxvalue);

	if(!Widget) Widget = new QLabel;
	Widget->setFixedSize(bgpixmap.width(), bgpixmap.height());

	if(!SLWidget)
	{
		int mlx = x+(bgpixmap.width()-slpixmap.width())/2;

		printf("mlc>\n");
		SLWidget = new MovingLabel(mlx,y,mlx,y+bgpixmap.height()-slpixmap.height(),this);
		printf("mlc<\n");
	}
	SLWidget->setFixedSize(slpixmap.width(), slpixmap.height());
	SLWidget->setBackgroundPixmap(slpixmap);

	valuePort = lookupPort(desc,"value");
}

void Gui_SLIDER::setParent(QWidget *parent, QBoxLayout *layout)
{
	static QBitmap a,b;
	if(Widget)
	{
		printf("recreating widget...\n");
		Widget->recreate(parent,0,QPoint(x,y));
		SLWidget->recreate(parent,0,
					QPoint(x+(bgpixmap.width()-slpixmap.width())/2,y));
		if(layout)
		{
			printf("adding widget to layout\n");
			layout->addWidget(Widget);
		}
		if(bgpixmap.height())
		{
			a = *bgpixmap.mask();
			Widget->setMask(a);
			Widget->setBackgroundPixmap(bgpixmap);
		}
		else
			printf("missing slider background pixmap!\n");
		if(slpixmap.height())
		{
			b = *slpixmap.mask();
			SLWidget->setMask(b);
			SLWidget->setBackgroundPixmap(slpixmap);
		}
		else
			printf("missing slider pixmap!\n");
		printf("ok.\n");
	}
	ParentWidget = parent;
}

void Gui_SLIDER::finalize()
{
	emitSignal(valuePort,maxvalue);
}

void Gui_SLIDER::start()
{
	Widget->show();
	SLWidget->show();
}

void Gui_SLIDER::sendPosChange(float x, float y)
{
	float sly = ((1-y) * (maxvalue - minvalue))+minvalue;
//	printf("new slider position %f  -> mapped to %f\n",y,sly);
	emitSignal(valuePort,sly);
}
