#ifndef __SCOMPONENT_H__
#define __SCOMPONENT_H__

#include <qscrollview.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <list>

class ModulePort;

class StructureCanvas {
public:
	virtual void redrawRect(int x, int y, int width, int height) = 0;
};

class StructureComponent {
protected:
	StructureCanvas *canvas;

public:
	StructureComponent(StructureCanvas *canvas);
	virtual ~StructureComponent();

	// type

	enum ComponentType { ctModule, ctPort };
	virtual ComponentType type() = 0;
	
	// TODO: connection & autorouter

	virtual ModulePort *portAt(int segment, int x, int y) = 0;
	virtual void dumpPorts(list<ModulePort *>& ports) = 0;

	// visibility
	virtual void hide() = 0;
	virtual void show() = 0;
	virtual bool visible() = 0;

	// drawing
	virtual bool drawNeedsBackground(int segment) = 0;
	virtual void drawSegment(QPainter *dest, int cellsize, int segment) = 0;
	virtual QPixmap *pixmap() = 0;
	virtual const char *name() = 0;

	// space checking & positioning
	virtual bool move(int x, int y) = 0;
	virtual int width() = 0;
	virtual int height() = 0;
	virtual int x() = 0;
	virtual int y() = 0;

	// selection
	virtual bool selected() = 0;
	virtual void setSelected(bool newselection) = 0;

	// creation and destruction are handled via standard
	// constructors/destructors
};

#endif
