
/*
** Cleaner version of midisend, by David G. Slomin.
** Released to the Public Domain, 1/25/99.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>

#include "midibus.h"

extern "C" 
{
#include "midimsg.h"
}

int main(int argc, char *argv[])
{
   /* 
   ** CORBA initialization.
   */

   CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "mico-local-orb");
   CORBA::BOA_var boa = orb->BOA_init(argc, argv, "mico-local-boa");

   CORBA::Object_var obj = orb->bind("IDL:MidiChannel:1.0", 
      "inet:localhost:8888");

   if (CORBA::is_nil(obj)) 
   {
      fprintf(stderr, "%s trouble: No midichannel CORBA object found; please start "
         "synth_server.\n",argv[0]);
      exit(EXIT_FAILURE);
   }

   MidiChannel_var midich = MidiChannel::_narrow(obj);

   /*
   ** MIDI input initialization.
   */

   int input_fd = -1, test = 0;

   switch (argc)
   {
      case 1:
         input_fd = open("/dev/midi", O_RDONLY);
         break;
      case 2:
         input_fd = ((strcmp(argv[1], "-") == 0) ? 0 : open(argv[1], O_RDONLY));
		 if(strcmp(argv[1],"--test") == 0) test = 1;
		 if(strcmp(argv[1],"--longtest") == 0) test = 2;
         break;
      default:
         fprintf(stderr, "Usage: %s [input-device-or-fifo]\n", argv[0]);
         fprintf(stderr, "where the input device is /dev/midi if "
            "unspecified\n");
         fprintf(stderr, "or stdin if you specify a dash\n");
         fprintf(stderr, "or test if you specify --test\n");
         exit(EXIT_FAILURE);
         break;
   }

   if(test)
   {
      unsigned long i,max=5000;
      if(test==2) max = 20000;
      for(i=0;i<max;i++)
      {
         midich->noteOn(0, 60+(i%12), 100);
         midich->noteOff(0, 60+(i%12));
      }
      exit(0);
   }

   if(input_fd == -1)
   {
      fprintf(stderr,"%s trouble: can't open input device!\n",argv[0]);
      exit(EXIT_FAILURE);
   }

   /*
   ** Main loop.
   */

   unsigned char msg[3];

   while (1)
   {
      midimsgRead(input_fd, msg);

      switch (midimsgGetMessageType(msg))
      {
         case MIDIMSG_NOTE_OFF:
            midich->noteOff(midimsgGetChannel(msg), midimsgGetPitch(msg));
            break;
         case MIDIMSG_NOTE_ON:
            midich->noteOn(midimsgGetChannel(msg), midimsgGetPitch(msg),
               midimsgGetVelocity(msg));
            break;
      }
   }
}

