#include "mbroker_impl.h"
#include "debug.h"

MB_ServerEntry::MB_ServerEntry(Arts::ArtsServer *server, Arts::ModuleInfo *info)
{
	this->server = server;
	this->info = info;
}

Arts::StringSeq* ModuleBroker_impl::publishedModules()
{
	Arts::StringSeq* seq = new Arts::StringSeq;
	list<MB_ServerEntry *>::iterator i;
	set<string> names;
	set<string>::iterator si;

	for(i=infolist.begin(); i != infolist.end(); i++)
	{
		Arts::ModuleInfo& mi = *((*i)->info);

		names.insert((const char *)mi.name);
	}

	unsigned long n = 0;
	seq->length(names.size());
	for(si=names.begin();si != names.end();si++)
		(*seq)[n++] = CORBA::string_dup((*si).c_str());

	return seq; 
}


void ModuleBroker_impl::publishModule( Arts::ArtsServer *server,
                                       const Arts::ModuleInfo& info )
{
	Arts::ModuleInfo *newinfo = new Arts::ModuleInfo(info);

	artsdebug("Broker::publishModule called:\n");
	artsdebug("  Server ID=%ld does registering: %s\n",
							server->ID(),(const char *)info.name);


	// FIXME: do name collision checking here!
	// two modules may have the same name, if they have exactly the same
	// interface... otherwise: complain

	_updateCount++;
	infolist.push_back(
		new MB_ServerEntry(Arts::ArtsServer::_duplicate(server),newinfo));
}


Arts::ModuleInfo* ModuleBroker_impl::lookupModule( const char* name )
{
	list<MB_ServerEntry *>::iterator i;

	for(i=infolist.begin(); i != infolist.end(); i++)
	{
		Arts::ModuleInfo& mi = *((*i)->info);
		if(strcmp(mi.name,name) == 0)
		{
			return(new Arts::ModuleInfo(mi));
		}
	}

	return 0; 	// not found
}


void
ModuleBroker_impl::removeModule( Arts::ArtsServer *server, const char* name )
{
  // add your implementation here
    // REMOVE  
    mico_throw(::CORBA::NO_IMPLEMENT());
    // REMOVE 

}

void
ModuleBroker_impl::serverShutdown( Arts::ArtsServer *server )
{
	list<MB_ServerEntry *>::iterator i;
	long sid = server->ID();

	for(i=infolist.begin(); i != infolist.end();)
	{
		if((*i)->server->ID() == sid)
		{
			delete (*i);
			infolist.erase(i);
			i = infolist.begin();
		}
		else i++;
	}
	artsdebug("Broker serverShutdown(ID %d) processed, %ld modules remaining\n",
                sid,infolist.size());
}

Arts::ArtsServerSeq* ModuleBroker_impl::whichServerProvides( char const *name )
{
	Arts::ArtsServerSeq *result = new Arts::ArtsServerSeq;

	list<MB_ServerEntry *>::iterator i;

	for(i=infolist.begin(); i != infolist.end(); i++)
	{
		Arts::ModuleInfo& mi = *((*i)->info);
		if(strcmp(mi.name,name) == 0)
		{
			unsigned long l = result->length();
			result->length(l+1);
			(*result)[l] = Arts::ArtsServer::_duplicate((*i)->server);
		}
	}
	return(result);
}

CORBA::Long ModuleBroker_impl::mkID()
{
	return(nextID++);
}

CORBA::Long ModuleBroker_impl::updateCount()
{
	return(_updateCount);
}

ModuleBroker_impl::ModuleBroker_impl()
{
	describe("ModuleBroker");

	_updateCount = 0;	
	nextID = 1;
}
